/*
 * This file is part of Sponge, licensed under the MIT License (MIT).
 *
 * Copyright (c) SpongePowered <https://www.spongepowered.org>
 * Copyright (c) contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.spongepowered.mod.data;

import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityList;
import net.minecraft.item.ItemMonsterPlacer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraftforge.fml.common.registry.EntityRegistry;
import org.spongepowered.api.data.DataTransactionResult;
import org.spongepowered.api.data.key.Keys;
import org.spongepowered.api.data.value.ValueContainer;
import org.spongepowered.api.data.value.immutable.ImmutableValue;
import org.spongepowered.api.data.value.mutable.Value;
import org.spongepowered.api.entity.EntityType;
import org.spongepowered.api.entity.EntityTypes;
import org.spongepowered.common.SpongeImpl;
import org.spongepowered.common.data.processor.common.AbstractSpongeValueProcessor;
import org.spongepowered.common.data.value.SpongeValueFactory;

import java.util.Optional;

public class SpawnableEntityTypeValueProcessor extends AbstractSpongeValueProcessor<ItemStack, EntityType, Value<EntityType>> {

    public SpawnableEntityTypeValueProcessor() {
        super(ItemStack.class, Keys.SPAWNABLE_ENTITY_TYPE);
    }

    @Override
    public int getPriority() {
        return 200;
    }

    @Override
    public Value<EntityType> constructValue(EntityType defaultValue) {
        return SpongeValueFactory.getInstance().createValue(Keys.SPAWNABLE_ENTITY_TYPE, defaultValue, EntityTypes.CREEPER);
    }

    @SuppressWarnings("unchecked")
    @Override
    public boolean set(ItemStack container, EntityType value) {
        final String name = (String) EntityList.field_75626_c.get(value.getEntityClass());
        if (EntityList.field_180126_g.containsKey(name)) {
            final int id = (int) EntityList.field_180126_g.get(name);
            if (EntityList.field_75627_a.containsKey(id)) {
                if (container.func_77942_o() && container.func_77978_p().func_74764_b("entity_name")) {
                    container.func_77978_p().func_82580_o("entity_name");
                }
                container.func_77964_b(id);
                return true;
            }
        } else {
            if (!container.func_77942_o()) {
                container.func_77982_d(new NBTTagCompound());
            }
            NBTTagCompound tag = container.func_77978_p();
            tag.func_74778_a("entity_name", name);
            final EntityRegistry.EntityRegistration registration = EntityRegistry.instance()
                    .lookupModSpawn((Class<? extends Entity>) value.getEntityClass(), false);
            container.func_77964_b(registration.getModEntityId());
            return true;
        }
        return false;
    }

    @Override
    public Optional<EntityType> getVal(ItemStack container) {
        final Class<?> entity = (Class<?>) EntityList.field_75625_b.get(ItemMonsterPlacer.getEntityName(container));
        for (EntityType type : SpongeImpl.getRegistry().getAllOf(EntityType.class)) {
            if (type.getEntityClass().equals(entity)) {
                return Optional.of(type);
            }
        }
        return Optional.empty();
    }

    @Override
    public ImmutableValue<EntityType> constructImmutableValue(EntityType value) {
        return constructValue(value).asImmutable();
    }

    @Override
    public DataTransactionResult removeFrom(ValueContainer<?> container) {
        if (this.supports(container)) {
            ItemStack item = (ItemStack) container;
            item.func_77964_b(0);
            if (item.func_77942_o() && item.func_77978_p().func_150297_b("entity_name", 8)) {
                item.func_77978_p().func_82580_o("entity_name");
            }
            return DataTransactionResult.successNoData();
        }
        return DataTransactionResult.failNoData();
    }
}
